local LE_EmoteList = ZO_Object:Subclass()

local _functionGetEmoteName

local _buttonTemplates = {
	"LE_EmoteTextButtonTemplate",
	"LE_EmoteDefaultButtonTemplate",
}

local function UpdateCategories(self)
	local function OnCategoryChanged(id)
		ZO_ScrollList_HideAllCategories(self.ListControl)
		ZO_ScrollList_ShowCategory(self.ListControl, id)
		ZO_ScrollList_ResetToTop(self.ListControl)
	end

	local function OnShowAll()
		ZO_ScrollList_HideAllCategories(self.ListControl)

		for i = #self.TempCategories, 1, -1  do
			ZO_ScrollList_ShowCategory(self.ListControl, self.TempCategories[i].ID)
		end

		ZO_ScrollList_ResetToTop(self.ListControl)
	end

	self.CategoryBox:ClearItems()
	table.sort(self.TempCategories, function(firstValue, secondValue) return firstValue.DisplayName < secondValue.DisplayName end)

	local firstEntry = self.CategoryBox:CreateItemEntry(GetString(SI_MARKETFILTERVIEW1), OnShowAll)
	self.CategoryBox:AddItem(firstEntry)

	for i, v in ipairs(self.TempCategories) do
		local entry = self.CategoryBox:CreateItemEntry(v.DisplayName, function()
			OnCategoryChanged(v.ID)
		end)
		self.CategoryBox:AddItem(entry)
	end

	self.CategoryBox:SelectFirstItem()
end

local function UpdateList(self, emoteList)
	ZO_ScrollList_Clear(self.ListControl)
	ZO_ClearNumericallyIndexedTable(self.TempCategories)
	local collectedCategories = {}

	for i, emote in ipairs(emoteList) do
		local categoryId = emote.CategoryID

		table.insert(self.DataList, ZO_ScrollList_CreateDataEntry(1, emote, categoryId))

		if not collectedCategories[categoryId] then
			collectedCategories[categoryId] = true

			table.insert(self.TempCategories, {
				DisplayName = LovelyEmotes.GetCategoryNameByID(categoryId),
				ID = categoryId,
			} )

			ZO_ScrollList_AddCategory(self.ListControl, categoryId)
		end
	end

	ZO_ScrollList_Commit(self.ListControl, self.DataList)
	UpdateCategories(self)
end

function LE_EmoteList:New()
	local emoteList = ZO_Object.New(self)
	emoteList:Initialize()
	return emoteList
end

function LE_EmoteList:Initialize()
	local savedAccountVariables = LovelyEmotes.SavedAccountVariables

	self.ParentControl = CreateControlFromVirtual(nil, GuiRoot, "LE_EmoteListControl")

	local searchControl = self.ParentControl:GetNamedChild("Search")

	self.ListControl = self.ParentControl:GetNamedChild("List")
	self.EditControl = searchControl:GetNamedChild("Box"):GetNamedChild("Edit")

	self.CategoryBox = ZO_ComboBox_ObjectFromContainer(self.ParentControl:GetNamedChild("CategoryBox"))
	self.CategoryBox:SetSortsItems(false)

	self.TempCategories = {}

	local function SetupCallback(control, data)
		if data.ID < 0 and LovelyEmotes.SavedAccountVariables.FavoriteCommandsShowDisplayName then
			control:SetText(data.DisplayName)
		else
			control:SetText(_functionGetEmoteName(data))
		end

		control:SetHandler("OnClicked", function(control, button) self.ListItemOnClick(button, data) end)

		local collectibleId = GetEmoteCollectibleId(data.Index)

		if not LovelyEmotes.SavedAccountVariables.HighlightLockedEmotes or not collectibleId or IsCollectibleUnlocked(collectibleId) then
			control:SetEnabled(true)
			control:SetNormalFontColor(LovelyEmotes.GetEmoteTextColor(data))
		else
			control:SetEnabled(false)
		end
	end

	ZO_ScrollList_AddDataType(self.ListControl, 1, _buttonTemplates[savedAccountVariables.EmoteListButtonDesign], 25, SetupCallback)
	self.DataList = ZO_ScrollList_GetDataList(self.ListControl)

	local startSearch = function()
		self:SearchByTag(self.EditControl:GetText())
		self.EditControl:LoseFocus()
	end

	self.EditControl:SetHandler("OnEnter", startSearch)
	searchControl:GetNamedChild("Button"):SetHandler("OnClicked", startSearch)
	searchControl:GetNamedChild("ResetButton"):SetHandler("OnClicked", function() self:ResetList() end)
end

function LE_EmoteList:SearchByTag(text)
	if text == nil or text == "" then
		self:ResetList()
		return
	end

	text = string.lower(text)
	local newEmoteList = {}

	for i,v in ipairs(LovelyEmotes.AvailableEmotes) do
		if string.match(v.TagString, text) then
			table.insert(newEmoteList, v)
		end
	end

	UpdateList(self, newEmoteList)
end

function LE_EmoteList:ResetList()
	UpdateList(self, LovelyEmotes.AvailableEmotes)
	self.EditControl:Clear()
end

function LE_EmoteList:SelectCategoryShowAll()
	self.CategoryBox:SelectFirstItem()
end

function LE_EmoteList:RefreshVisible()
	ZO_ScrollList_RefreshVisible(self.ListControl)
end

function LE_EmoteList:SetParent(newParentControl, offsetY, onClickFunc)
	self.ParentControl:ClearAnchors()
	self.ParentControl:SetParent(newParentControl)
	self.ParentControl:SetAnchor(TOPLEFT, newParentControl, TOPLEFT, 0, LE_Const_MainWindowBaseHeight + offsetY)
	self.ParentControl:SetAnchor(BOTTOMRIGHT, newParentControl, BOTTOMRIGHT, 0, 0)

	self.ListItemOnClick = onClickFunc
end

function LovelyEmotes_EmoteList_ShowSlashNames(value)
	if value then
		_functionGetEmoteName = LovelyEmotes.GetEmoteSlashName
	else
		_functionGetEmoteName = LovelyEmotes.GetEmoteDisplayName
	end
end

function LovelyEmotes_EmoteList_CreateNew()
	return LE_EmoteList:New()
end
