local _listControl
local _dataList

local _buttonTemplates = {
	"LE_EmoteTextButtonTemplate",
	"LE_EmoteDefaultButtonTemplate",
}

local function SetupList()
	ZO_ScrollList_Clear(_listControl)
	ZO_ScrollList_AddCategory(_listControl, 1)
	local personalities = LovelyEmotes.AvailablePersonalities

	for i, p in ipairs(personalities) do
		table.insert(_dataList, ZO_ScrollList_CreateDataEntry(1, p, 1))
	end

	table.sort(_dataList, function(firstValue, secondValue) return firstValue.data.Name < secondValue.data.Name end)
	ZO_ScrollList_Commit(_listControl, _dataList)
end

local function SetupCallback(control, data)
	if IsCollectibleActive(data.CollectibleId) then
		control:SetNormalFontColor(LOVELYEMOTES_PERSONALITY_ACTIVE_COLOR:UnpackRGBA())
	else
		control:SetNormalFontColor(ZO_NORMAL_TEXT:UnpackRGBA())
	end

	control:SetText(data.Name)
	control:SetHandler("OnClicked", function(control, button) UseCollectible(data.CollectibleId) end)
end

function LovelyEmotes.PersonalityList.Reset()
	ZO_ScrollList_ResetToTop(_listControl)
	SetupList()
end

local function RefreshVisible()
	if _listControl:IsControlHidden() then return end
	ZO_ScrollList_RefreshVisible(_listControl)
end
LovelyEmotes.PersonalityList.RefreshVisible = RefreshVisible

local function OpenFunction()
	local buttonsCount = #_dataList
	if buttonsCount > 14 then buttonsCount = 14 end

	LovelyEmotes.MainWindow.SetWindowHeight(buttonsCount * 25)
	_listControl:SetHidden(false)
	ZO_ScrollList_RefreshVisible(_listControl)
end

local function CloseFunction()
	_listControl:SetHidden(true)
end

function LovelyEmotes.PersonalityList.GetFunctions()
	return OpenFunction, CloseFunction
end

function LovelyEmotes.PersonalityList.Initialize()
	_listControl = LE_MainWindowControlPersonalityList

	ZO_ScrollList_AddDataType(_listControl, 1, _buttonTemplates[LovelyEmotes.SavedAccountVariables.EmoteListButtonDesign], 25, SetupCallback)
	_dataList = ZO_ScrollList_GetDataList(_listControl)

	SetupList()
end
